from ngsolve import *
from netgen.geom2d import unit_square
import time

Start = time.time()

# generate a triangular mesh of mesh-size 0.2
mesh = Mesh(unit_square.GenerateMesh(maxh=0.01))

# Hcurl-conforming finite element space
fes = HCurl(mesh, order=5, complex=True, dirichlet=[1,2,3,4])

# define trial- and test-functions
u = fes.TrialFunction()
v = fes.TestFunction()

# the right hand side
f = LinearForm(fes)
f += SymbolicLFI(CoefficientFunction((1j*y*(1-y),x*(1-x))) * v)

# the bilinear-form 
a = BilinearForm(fes, symmetric=True)
a += SymbolicBFI(curl(u)*curl(v)+u*v)
c = Preconditioner(a,"bddc")

a.Assemble()
f.Assemble()
c.Update()

gfu = GridFunction(fes)
res = f.vec.CreateVector()
res.data = f.vec - a.mat * gfu.vec

inverse= CGSolver(a.mat, c.mat, precision=10**(-4), maxsteps=100)
gfu.vec.data += inverse * res

print(time.time()-Start)